#version 150

// vsPosition was bound to attribute index 0 and vsColor was bound to attribute index 1
in vec3 vsPosition;
in vec3 vsNormal;
in vec4 vsColor;
in vec2 vsTextureUV;

// Our outputs the next shader (fragment) in the chain
out vec4 fsColor;
out vec2 fsTextureUV;
out vec4 shadowCoordinates[4];

out vec3 fsDirectionVS;
out vec3 fsPositionEye;
out vec3 fsNormalEye;
out vec3 fsNormalVS;
out vec3 fsNormalWS;
out vec3 fsSunDirectionVS;
out mat4 fsInvView;

uniform mat4 uObjectToProjection;
uniform mat4 uObjectToView;
uniform mat4 uObjectToWorld;
uniform mat4 uWorldToView;
uniform mat4 sunCameraView;
uniform mat4 sunCameraProjection[4];

//Expected to be a unit vector and in world space already. Pointing FROM origin TO the sun.
uniform vec4 sunLightDirection = vec4(-0.447766751, 0.373415232, 0.812444329, 0.422158539);
uniform vec4 shadowNormalOffset = vec4(0.05);

const mat4 bias = mat4(
	vec4(0.5, 0.0, 0.0, 0.5),
	vec4(0.0, 0.5, 0.0, 0.5),
	vec4(0.0, 0.0, 0.5, 0.5),
	vec4(0.0, 0.0, 0.0, 1.0)
);

void main(void)
{
	mat4 uWorldToSunView = sunCameraView;
	mat4 uSunViewToSunProjection[4] = sunCameraProjection;

	gl_Position = vec4(vsPosition, 1.0) * uObjectToProjection;

	fsNormalWS = vec3(normalize(vec4(vsNormal, 0.0) * uObjectToWorld));
	fsColor = vsColor;
	fsTextureUV = vsTextureUV;

	mat4 modelSunView = uObjectToWorld * uWorldToSunView;

	fsPositionEye = (vec4(vsPosition, 1.0) * uObjectToView).xyz;
	fsNormalEye = fsNormalWS * mat3(uWorldToView);

	fsNormalVS = fsNormalWS * mat3(uWorldToView);
	fsSunDirectionVS = (vec4(vec3(sunLightDirection), 0.0) * uWorldToView).xyz;

	fsInvView = inverse(uWorldToView);

	// vec3 mirror_dir = reflect(-view_dir, normal);
	// mirror_dir = vec3(inverse(uWorldToView) * vec4(mirror_dir, 0.0));

	// mat3 normalMatrix = uObjectToWorld; /* this will break for non-uniform scaling */
	// vec3 normalW = normalize(vsNormal * normalMatrix);
	// vec3 tangentW = normalize(vec3(uObjectToWorld * vec4(a_tangent.xyz, 0.0)));
	// vec3 bitangentW = cross(normalW, tangentW) * a_tangent.w;
	// fsTbnMatrix = mat3(tangentW, bitangentW, normalW);

	vec4 wpos = vec4(vsPosition, 1.0) * uObjectToWorld;
	vec4 vpos = wpos * uWorldToView;
	fsDirectionVS = -vpos.xyz;

	for (int i = 0; i < 4; i++)
	{
		float shadowOffset = max(0.0, dot(fsNormalWS, sunLightDirection.xyz)) * shadowNormalOffset[i];
		vec4 positionOffset = vec4(vsPosition + fsNormalWS * shadowOffset, 1.0f) * modelSunView;
		shadowCoordinates[i] = positionOffset * uSunViewToSunProjection[i] * bias;
		//shadowCoordinates[i] = vec4(vsPosition + vsNormal * 0.01, 1.0) * uObjectToWorld * uWorldToSunView * uSunViewToSunProjection[i] * bias;
		//shadowCoordinates[i] = vec4(vsPosition, 1.0) * uObjectToWorld * uWorldToSunView * uSunViewToSunProjection[i] * bias;
	}
}
